//----------------------------------------------------------------
// fichier .cpp des classes des robots
//
#include "stdafx.h"
#include "mfc_gl.h"

#include "mfc_glDoc.h"
#include "mfc_glView.h"
#include "font.h"

#include <math.h>


const double M_PI=3.1415926535898;


//-------------------------------------------------------------------------------------
//  TRoue
//-------------------------------------------------------------------------------------
TRoue::TRoue()
{
	M=0.01; // masse 0.01 kg / Roue non motrice
	r=0.01; R=0.05;
	Reset();					// initialisation des grandeurs de la roue  0
}
//-------------------------------------------------------------------------------------
void TRoue::Dessine(double posx, double posy, int type)
{
	glPushMatrix();
		glTranslatef ( (GLfloat)posx, (GLfloat)posy, 0);
		if (Delta!=0)	glRotatef ((GLfloat)(Delta*180.0/M_PI), (GLfloat)0.0, (GLfloat)0.0, (GLfloat)1.0);
		glRotatef (-(GLfloat)(Theta*180.0/M_PI), (GLfloat)1.0, (GLfloat)0.0, (GLfloat)0.0);
		glColor3d(0,0.3,0.5);
		if (type==RoueTorique)	glCallList(RoueTorique);
				else								glCallList(Roue);
	glPopMatrix();
}
//-------------------------------------------------------------------------------------
void TRoue::Reset()
{
	W=0; Theta=0;			// position et vitesse angulaires instantanes de la roue
	Delta=0;					// le braquage de la roue se fera uniquement pour le Robot4
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//  TRobot2
//-------------------------------------------------------------------------------------
TRobot2::TRobot2(double APosX0, double APosY0, int num)
{
	Type=TypeRobot2;
	Numero=num;
	Selected=false;
	RAvD=new TRoue();
	RAvG=new TRoue();
//	RArG, RArD;	// les autres roues ?
	M=0.1 + RAvD->M + RAvG->M; // masse 1kg + les roues
	Ly=0.18; Lx=0.15; e=0.01; Gabarit=0.15;
	Lxc=0.12, Lyc=0.22, Lzc=0.03;
	PosX0=APosX0; PosY0=APosY0;
	kech=1.0;					// echelle de visualisation
	Reset();					// initialisation des grandeurs
}
//-------------------------------------------------------------------------------------
TRobot2::~TRobot2()
{
	delete 	RAvD;
	delete 	RAvG;
}
//-------------------------------------------------------------------------------------
// se dessine
void TRobot2::Dessine()
{
double xMR, yMR, zMR;
char buffer[50];
	xMR=kech*PosX;
	zMR=kech*RAvD->R;
	yMR=kech*PosY;
glPushMatrix();
	glTranslated((GLfloat)xMR, (GLfloat)yMR, (GLfloat)zMR);
	glRotatef ((GLfloat)(DeltaR*180.0/M_PI), (GLfloat)0.0, (GLfloat)0.0, (GLfloat)1.0);
	glScalef((GLfloat)kech,(GLfloat)kech,(GLfloat)kech);	// changement d'echelle pour agrandir le vhicule par rapport  la ralit
  // essieux Av et Ar
	glPushMatrix();
		glColor3d(0.8,0,0.7);
		glTranslatef (-(GLfloat)Lx/2, -(GLfloat)Ly/2, 0 );
		glCallList(TigeD);
		glTranslatef (0, (GLfloat)Ly, 0);
		glCallList(TigeD);
	glPopMatrix();
	// chaque roue se dessine elle mme (position et type (tore ou disque))
	RAvG->Dessine(-Lx/2, Ly/2, RoueTorique);
	RAvD->Dessine( Lx/2, Ly/2, Roue);
	// chassis
  glEnable(GL_TEXTURE_2D);
	glCallList(Robot2Chassis);
  glDisable(GL_TEXTURE_2D);
// ecrit l'indice du robot
  if (Selected)	glColor3d(0.95,0,0);
		else	glColor3d(0.0,0.0,0.8);
  glRasterPos3d(0, 0, 0.05);
	itoa( Numero, buffer, 10);
  printString(buffer);

glPopMatrix();
}
//-------------------------------------------------------------------------------------
void TRobot2::IntegreEquations(double dt)
{
// integre ses equation mcanique (RFD) ds le temps
// pour simuler le deplacement
// ici exemple du robot 2 avec vitesse fixe
	VitY=0.1;
	PosY+=VitY*dt;
	Vit=sqrt(VitX*VitX+VitY*VitY);
  RAvD->Theta+=Vit/RAvD->R*dt;
  RAvG->Theta+=Vit/RAvG->R*dt;
}
//-------------------------------------------------------------------------------------
void TRobot2::Pilotage(double t)
{
}
//-------------------------------------------------------------------------------------
void TRobot2::Reset()
{
  VitX=0; VitY=0; Vit=0;
	DeltaR=0.0;	W=0.0;			// angle et vitesse de rotation (repere x'y' du robot % xy de l'environnement physique)
	PosX=PosX0; PosY=PosY0;	// se remet  la position initiale sauve dans PosX0, PosY0
  RAvD->Reset();					// Remet les grandeurs des roues  0
  RAvG->Reset();
}
//-------------------------------------------------------------------------------------

